#include <sys/queue.h>
#include <assert.h>
#include <string.h>
#include <sys/stat.h>		/* for struct srcfile */

#include "txt.h"
#include "miniobj.h"


#define AAP(ptr)	assert((ptr) != NULL)

#define INCOMPL() \
	do { \
		assert(0 == __LINE__); \
		/*lint -e527*/ \
		exit(1); \
		/*lint +e527*/ \
	} while (0)



/*********************************************************************
 * chunks of source
 */

struct chunk;

typedef void c_quotef_f(FILE *, struct chunk *, txt t);

struct chunk_method {
	const char		*name;
	c_quotef_f		*quote;
};

struct chunk {
	txt			*t;
	void			*priv;
	struct chunk_method	*f;
	struct tokl		*tokl;	/* first lex this chunk */
};


/*********************************************************************
 * srcfile.c
 */

struct chunk *LoadSrcFd(const char *name, int fd);
struct chunk *LoadSrcFile(const char *pfx, const char *name);
struct chunk *LoadSrcFileTxt(const char *pfx, txt name);

#if 0
/*********************************************************************
 * PreProc tokens
 */

enum e_ptok {
#define PTOK(a, b)	T_ ## a = b,
#include "ptok.h"
#undef PTOK
	SOME_RANDOM_PLACE_HOLDER
};

struct ptok {
	txt				body;
	struct ptok			*ref;
	uint8_t				tok;
	uint8_t				flags;
};
#define FLAG_ESCAPED_NL			(1<<0)

struct tokl {
	struct chunk			*chunk;
	unsigned			nptok;	/* space for */
	unsigned			uptok;  /* used */
	struct ptok			*ptok;
};

/*********************************************************************
 * lex.c
 */

struct tokl *Lex(struct chunk *c, unsigned xline);

#endif

/*********************************************************************
 * ktype.c
 */

#define CT_HEX		(1<<0)
#define CT_DIGIT	(1<<1)
#define CT_OCTAL	(1<<2)
#define CT_ALPHA	(1<<3)
#define CT_UNDER	(1<<4)
#define CT_SIGN		(1<<5)
#define CT_HWHITE	(1<<6)
#define CT_VWHITE	(1<<7)
#define CT_PPNEXP	(1<<8)

#define ct_hex(c)	(ct[(int)(c)] & CT_HEX)
#define ct_octal(c)	(ct[(int)(c)] & CT_OCTAL)
#define ct_digit(c)	(ct[(int)(c)] & CT_DIGIT)
#define ct_ident1(c)	(ct[(int)(c)] & (CT_ALPHA | CT_UNDER))
#define ct_ident(c)	(ct[(int)(c)] & (CT_ALPHA | CT_DIGIT | CT_UNDER))
#define ct_sign(c)	(ct[(int)(c)] & CT_SIGN)
#define ct_white(c)	(ct[(int)(c)] & (CT_HWHITE | CT_VWHITE))
#define ct_vwhite(c)	(ct[(int)(c)] & CT_VWHITE)
#define ct_ppnexp(c)	(ct[(int)(c)] & CT_PPNEXP)

extern const unsigned short ct[256];

#if 0

/*********************************************************************
 * hash.c
 */
void HashDirective(struct tokl *tl, struct ptok *ptb, struct ptok *pte);
extern int suppress;

/*********************************************************************
 * ptok.c
 */
void Quote(FILE *f, struct chunk *c, txt t);
void dumptokl(struct tokl *tl);
void dumpptok(struct ptok *pt);
void dumpptoks(struct ptok *pb, struct ptok *pe);

#if  0
/*********************************************************************
 *********************************************************************
 *********************************************************************
 *********************************************************************
 *********************************************************************
 * Our text-chunks
 */

struct chunk {
	txt			body;
	const void		*priv;
	const char		*kind;
	TAILQ_ENTRY(chunk)	list;
	struct tokl		*tokl;
	TAILQ_HEAD(,pptok)	tokens;
};

#define ACPTR(chunk, ptr)  do {				\
		assert((ptr) >= (chunk)->body.b);	\
		assert((ptr) <= (chunk)->body.e);	\
	} while (0)


TAILQ_HEAD(chunks, chunk);
struct pptok {
	txt				body;
	int 				tok;
	TAILQ_ENTRY(pptok)		list;
	TAILQ_ENTRY(pptok)		mlist;
	struct macro			*macro;
	struct macrodef			*def;
	struct pptok			*ptr;
	unsigned 			flags;
	struct chunk			*chunk;
	unsigned			line;
	struct macrodef			*mdef;
	int				nl;
};

TAILQ_HEAD(tokenhead, pptok);



/*********************************************************************
 * lex.c
 */

void PpLex(struct chunk *cin, unsigned xline);
struct tokl *TxtLex(txt cin, unsigned xline);
void PpEof(struct chunk *c);
struct tokl *Alloc_TokL(struct tokl *p, int n);
void PPtok(struct ptok *pt);

#endif
#endif
