/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: grammar.y,v 1.9 2005/10/31 08:01:12 phk Exp $
 *
 */

%{
#include <stdio.h>
#include <string.h>

#include "defs.h"
%}


%union	{
	char			str[1024];
	struct info		*info;
	struct infolist		*infolist;
}

%token TWORD TINSTANCE TNODE TEXTERN
%token TINT TUINT TSTRING TENUM TWORD TREAL
%token <str> ID
%token BOPEN BCLOSE
%token <str> STRING
%type <str> type
%type <str> entry
%type <str> enum
%type <infolist> tid_list
%type <infolist> type_list
%type <infolist> type_list_e
%type <info> leaf
%type <infolist> info_list
%type <infolist> leaf_list
%%

list		: entry
		| list entry
;

tid_list	: ID {
			$$ = new_infolist();
			append_infolist($$, $1, NULL);
		}
		| tid_list ID {
			$$ = $1;
			append_infolist($$, $2, NULL);
		}
;

enum		: TENUM BOPEN tid_list BCLOSE {
			strcpy($$, emit_enum($3));
		}
;

info_list	: leaf {
			$$ = new_infolist();
			if ($1 != NULL)
				TAILQ_INSERT_TAIL($$, $1, list);
		}
		| entry {
			$$ = new_infolist();
			append_infolist($$, "ENTRY", $1);
		}
		| info_list leaf {
			$$ = $1;
			if ($2 != NULL)
				TAILQ_INSERT_TAIL($$, $2, list);
		}
		| info_list entry {
			$$ = $1;
			append_infolist($$, "ENTRY", $2);
		}
;

leaf_list	: leaf {
			$$ = new_infolist();
			if ($1 != NULL)
				TAILQ_INSERT_TAIL($$, $1, list);
		}
		| leaf_list leaf {
			$$ = $1;
			if ($2 != NULL)
				TAILQ_INSERT_TAIL($$, $2, list);
		}
;

leaf		: ID STRING {
			$$ = new_info($1, $2);
		}
		| ID ID {
			$$ = new_info($1, $2);
		}
;

type		: TUINT {strcpy($$, "UINT"); }
		| TINT {strcpy($$, "INT"); }
		| enum {strcpy($$, $1); }
		| TSTRING {strcpy($$, "STRING"); }
		| TREAL {strcpy($$, "REAL"); }
		| TWORD {strcpy($$, "WORD"); }
;

type_list	: {
			$$ = new_infolist();
		}
		| type {
			$$ = new_infolist();
			append_infolist($$, $1, NULL);
		}
		| type_list type {
			$$ = $1;
			append_infolist($$, $2, NULL);
		}
;

type_list_e	: type_list {
			$$ = $1;
		}
		| type_list TELLIPSIS {
			$$ = $1;
			append_infolist($$, "ELLIPSIS", NULL);
		}
;

entry 		: TNODE ID BOPEN info_list BCLOSE {
			strcpy($$, emit_node($2, $4));
		}
		| TWORD ID type_list_e BOPEN leaf_list BCLOSE {
			strcpy($$, emit_word($2, $3, $5));
		}
		| TINSTANCE ID type BOPEN info_list BCLOSE {
			strcpy($$, emit_instance($2, $3, $5));
		}
		| TEXTERN ID ID BOPEN info_list BCLOSE {
			strcpy($$, emit_extern($2, $3, $5));
		}
		| TIMPORT ID BOPEN info_list BCLOSE {
			strcpy($$, emit_import($2, $4));
		}
;

