/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: lexer.l,v 1.10 2005/10/31 08:01:12 phk Exp $
 *
 */

%{
#include "stdio.h"
#include "string.h"
#include "grammar.h"
#include "sys/queue.h"

#include "defs.h"


static char string_buf[1000];
static char *string_buf_ptr = NULL;
%}

DIGIT    [0-9]
ID       [a-z][a-z0-9_]*

%x instring

%%

^#.*\n	{ lineno++; };
\n	{ lineno++; };

WORD	{ return TWORD; }
INSTANCE	{ return TINSTANCE; }
UINT	{ return TUINT; }
INT	{ return TINT; }
ENUM	{ return TENUM; }
STRING	{ return TSTRING; }
REAL	{ return TREAL; }
NODE	{ return TNODE; }
EXTERN	{ return TEXTERN; }
IMPORT	{ return TIMPORT; }
"..."	{ return TELLIPSIS; }

"{"	{ return BOPEN; }

"}"	{ return BCLOSE; }

{ID}    {
	strcpy(yylval.str, yytext);
	return (ID); 
	}

\"	string_buf_ptr = string_buf; BEGIN(instring);

<instring>\"	{
	BEGIN(INITIAL);
	*string_buf_ptr = '\0';
	strcpy(yylval.str, string_buf);
	return (STRING);
	}

<instring>\n	{
	printf("unterminated string\n");
	}

<instring>\\[0-7]{1,3}	{
                  /* octal escape sequence */
                   int result;

                   (void) sscanf( yytext + 1, "%o", &result );

                   if ( result > 0xff )
                           /* error, constant is out-of-bounds */

                   *string_buf_ptr++ = result;

	}

<instring>\\[0-9]+ {
                   /* generate error - bad escape sequence; something
                    * like '\48' or '\0777777'
                    */
                   }

<instring>\\n  *string_buf_ptr++ = '\n';
<instring>\\t  *string_buf_ptr++ = '\t';
<instring>\\r  *string_buf_ptr++ = '\r';
<instring>\\b  *string_buf_ptr++ = '\b';
<instring>\\f  *string_buf_ptr++ = '\f';

<instring>\\(.|\n)  *string_buf_ptr++ = yytext[1];

<instring>[^\\\n\"]+        {
                   char *yptr = yytext;

                   while ( *yptr )
                           *string_buf_ptr++ = *yptr++;
                   }



[ \t]+          /* eat up whitespace */

.	{
	printf( "Unrecognized character: %s near line %d\n", yytext, lineno );
	}

%%

